package part1

import (
	"bufio"
	"fmt"
	"io"
	"strconv"
	"strings"
)

func Calibrate(r io.Reader) (uint, error) {
	scanner := bufio.NewScanner(r)

	var sum uint

	for scanner.Scan() {
		line := scanner.Text()

		calVal, err := makeCalibrationValue(strings.NewReader(line))
		if err != nil {
			return 0, fmt.Errorf("cannot make calibration value from '%s': %w", line, err)
		}

		sum += uint(calVal)
	}

	return sum, nil
}

func makeCalibrationValue(r io.Reader) (uint8, error) {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanRunes)

	parts := make([]rune, 0)
	for scanner.Scan() {
		text := scanner.Bytes()

		switch text[0] {
		default:
			continue
		case '1', '2', '3', '4', '5', '6', '7', '8', '9':
			parts = append(parts, rune(text[0]))
		}
	}

	digit := fmt.Sprintf("%c%c", parts[0], parts[len(parts)-1])

	result, err := strconv.ParseUint(digit, 10, 8)
	if err != nil {
		return 0, fmt.Errorf("cannot parse '%s' as calibration value number: %w", digit, err)
	}

	return uint8(result), nil
}
