package part2

import (
	"bufio"
	"fmt"
	"io"
	"strconv"
	"strings"
)

func Calibrate(r io.Reader) (uint, error) {
	scanner := bufio.NewScanner(r)

	var sum uint

	for scanner.Scan() {
		line := scanner.Text()

		calVal, err := makeCalibrationValue(strings.NewReader(line))
		if err != nil {
			return 0, fmt.Errorf("cannot make calibration value from '%s': %w", line, err)
		}

		sum += uint(calVal)
	}

	return sum, nil
}

func makeCalibrationValue(r io.Reader) (uint8, error) {
	line, err := io.ReadAll(r)
	if err != nil {
		return 0, fmt.Errorf("cannot fetch reader bytes: %w", err)
	}

	parts := [2]rune{0, 0}

	// NOTE(toby3d): first left
	for i := 0; i < len(line); i++ {
		if d := stringToDigit(string(line[i:])); d != 0 {
			parts[0] = d

			break
		}
	}

	// NOTE(toby3d): last right
	for i := len(line) - 1; 0 <= i; i-- {
		if d := stringToDigit(string(line[i:])); d != 0 {
			parts[1] = d

			break
		}
	}

	digit := fmt.Sprintf("%c%c", parts[0], parts[1])

	result, err := strconv.ParseUint(digit, 10, 8)
	if err != nil {
		return 0, fmt.Errorf("cannot parse '%s' as calibration value number: %w", digit, err)
	}

	return uint8(result), nil
}

func stringToDigit(str string) rune {
	switch str[0] {
	case '1', '2', '3', '4', '5', '6', '7', '8', '9':
		return rune(str[0])
	}

	for name, digit := range map[string]rune{
		"one":   '1',
		"two":   '2',
		"three": '3',
		"four":  '4',
		"five":  '5',
		"six":   '6',
		"seven": '7',
		"eight": '8',
		"nine":  '9',
	} {
		if !strings.HasPrefix(str, name) {
			continue
		}

		return digit
	}

	return 0
}
