package part2

import (
	"bufio"
	"bytes"
	"io"
	"slices"
)

type (
	Sequence []Step

	Step []byte

	Lens struct {
		label  []byte
		length int
	}
)

func (s Step) Sum() uint {
	var result uint

	for i := range s.Hash() {
		result += uint(s[i])
		result *= 17
		result = result % 256
	}

	return result
}

func (s Step) IsRemove() bool {
	return bytes.IndexByte(s, '-') != -1
}

func (s Step) IsMove() bool {
	return bytes.IndexByte(s, '=') != -1
}

func (s Step) Hash() []byte {
	return s[:max(bytes.IndexByte(s, '-'), bytes.IndexByte(s, '='))]
}

func (s Step) Length() int {
	index := max(bytes.IndexByte(s, '-'), bytes.IndexByte(s, '='))
	if index == len(s)-1 {
		return 0
	}

	return int(s[index+1] - '0')
}

func (s Sequence) Sum() uint64 {
	var boxes [256][]Lens

	for i := range s {
		box := &boxes[int(s[i].Sum())]
		index := slices.IndexFunc(*box, func(l Lens) bool {
			return bytes.Equal(l.label, s[i].Hash())
		})

		switch {
		default:
			continue
		case s[i].IsRemove() && index != -1:
			*box = slices.Delete(*box, index, index+1)
		case s[i].IsMove():
			if index != -1 {
				(*box)[index] = Lens{
					label:  s[i].Hash(),
					length: s[i].Length(),
				}
			} else {
				*box = append(*box, Lens{
					label:  s[i].Hash(),
					length: s[i].Length(),
				})
			}
		}
	}

	var result int
	for i := range boxes {
		for j := range boxes[i] {
			result += (i + 1) * (j + 1) * boxes[i][j].length
		}
	}

	return uint64(result)
}

func ParseSequence(r io.Reader) Sequence {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanBytes)

	seq := make(Sequence, 0)
	step := make(Step, 0)

	for scanner.Scan() {
		b := scanner.Bytes()[0]
		switch b {
		case ',', '\n':
			seq = append(seq, step)
			step = make(Step, 0)

			continue
		}

		step = append(step, b)
	}

	return seq
}

func (s Step) GoString() string {
	return string(s)
}
