package part2_test

import (
	"fmt"
	"strings"
	"testing"

	"source.toby3d.me/toby3d/adventofcode/internal/day15/part2"
)

func TestStep_Hash(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		expect string
		input  part2.Step
	}{
		"rn=1": {input: part2.Step("rn=1"), expect: "rn"},
		"cm-":  {input: part2.Step("cm-"), expect: "cm"},
		"qp=3": {input: part2.Step("qp=3"), expect: "qp"},
		"cm=2": {input: part2.Step("cm=2"), expect: "cm"},
		"qp-":  {input: part2.Step("qp-"), expect: "qp"},
		"pc=4": {input: part2.Step("pc=4"), expect: "pc"},
		"ot=9": {input: part2.Step("ot=9"), expect: "ot"},
		"ab=5": {input: part2.Step("ab=5"), expect: "ab"},
		"pc-":  {input: part2.Step("pc-"), expect: "pc"},
		"pc=6": {input: part2.Step("pc=6"), expect: "pc"},
		"ot=7": {input: part2.Step("ot=7"), expect: "ot"},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			if actual := string(tc.input.Hash()); actual != tc.expect {
				t.Errorf("want '%s', got '%s'", tc.expect, actual)
			}
		})
	}
}

func TestStep_Length(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		input  part2.Step
		expect int
	}{
		"rn=1": {input: part2.Step("rn=1"), expect: 1},
		"cm-":  {input: part2.Step("cm-"), expect: 0},
		"qp=3": {input: part2.Step("qp=3"), expect: 3},
		"cm=2": {input: part2.Step("cm=2"), expect: 2},
		"qp-":  {input: part2.Step("qp-"), expect: 0},
		"pc=4": {input: part2.Step("pc=4"), expect: 4},
		"ot=9": {input: part2.Step("ot=9"), expect: 9},
		"ab=5": {input: part2.Step("ab=5"), expect: 5},
		"pc-":  {input: part2.Step("pc-"), expect: 0},
		"pc=6": {input: part2.Step("pc=6"), expect: 6},
		"ot=7": {input: part2.Step("ot=7"), expect: 7},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			if actual := tc.input.Length(); actual != tc.expect {
				t.Errorf("want %d, got %d", tc.expect, actual)
			}
		})
	}
}

func TestStep_IsRemove(t *testing.T) {
	t.Parallel()

	if actual := part2.Step("qp-").IsRemove(); !actual {
		t.Errorf("want %t, got %t", true, actual)
	}
}

func TestStep_IsMove(t *testing.T) {
	t.Parallel()

	if actual := part2.Step("rn=1").IsMove(); !actual {
		t.Errorf("want %t, got %t", true, actual)
	}
}

func Example() {
	input := "rn=1,cm-,qp=3,cm=2,qp-,pc=4,ot=9,ab=5,pc-,pc=6,ot=7\n"
	seq := part2.ParseSequence(strings.NewReader(input))

	fmt.Println(seq.Sum())
	// Output: 145
}
