package part1

import (
	"bufio"
	"fmt"
	"io"
	"strconv"
	"strings"
)

type (
	Bag struct {
		Red   uint8
		Green uint8
		Blue  uint8
	}

	Game struct {
		Rounds []Round
		ID     uint8
	}

	Round struct {
		Red   uint8
		Green uint8
		Blue  uint8
	}
)

func ParseGame(str string) (*Game, error) {
	parts := strings.Split(str, ": ")
	if len(parts) != 2 {
		return nil, fmt.Errorf("bad input format: want 'Game 0: 0 color; 0 color, 0 color...', got '%s'", str)
	}

	id, err := strconv.ParseUint(strings.TrimPrefix(parts[0], "Game "), 10, 8)
	if err != nil {
		return nil, fmt.Errorf("cannot parse game id: %w", err)
	}

	rawRounds := strings.Split(parts[1], "; ")
	rounds := make([]Round, len(rawRounds))

	for i := range rawRounds {
		for _, record := range strings.Split(rawRounds[i], ", ") {
			recordParts := strings.Split(record, " ")
			if len(recordParts) != 2 {
				return nil, fmt.Errorf("cannot split record '%s' on cound and color", record)
			}

			count, err := strconv.ParseUint(recordParts[0], 10, 8)
			if err != nil {
				return nil, fmt.Errorf("cannot parse '%s' as cubes cound: %w", recordParts[0], err)
			}

			switch recordParts[1] {
			default:
				continue
			case "blue":
				rounds[i].Blue += uint8(count)
			case "green":
				rounds[i].Green += uint8(count)
			case "red":
				rounds[i].Red += uint8(count)
			}
		}
	}

	return &Game{
		ID:     uint8(id),
		Rounds: rounds,
	}, nil
}

func (b Bag) Cubes() uint8 {
	return b.Blue + b.Green + b.Red
}

func (r Round) Cubes() uint8 {
	return r.Blue + r.Green + r.Red
}

func (r Round) IsPossible(bag Bag) bool {
	return r.Cubes() <= bag.Cubes() &&
		r.Blue <= bag.Blue &&
		r.Green <= bag.Green &&
		r.Red <= bag.Red
}

func (g Game) IsPossible(bag Bag) bool {
	for i := range g.Rounds {
		if g.Rounds[i].IsPossible(bag) {
			continue
		}

		return false
	}

	return true
}

func GetSecretNumber(r io.Reader, target Bag) (uint, error) {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanLines)

	var result uint
	for scanner.Scan() {
		line := scanner.Text()
		game, err := ParseGame(line)
		if err != nil {
			return 0, fmt.Errorf("cannot parse game from line '%s': %w", line, err)
		}

		if !game.IsPossible(target) {
			continue
		}

		result += uint(game.ID)
	}

	return result, nil
}
