package part2

import (
	"bufio"
	"fmt"
	"io"
	"strconv"
	"strings"
)

type Minimum struct {
	Red   uint8
	Green uint8
	Blue  uint8
}

func ParseMinimum(str string) (*Minimum, error) {
	parts := strings.Split(str, ": ")
	if len(parts) != 2 {
		return nil, fmt.Errorf("bad input format: want 'Game 0: 0 color; 0 color, 0 color...', got '%s'", str)
	}

	result := new(Minimum)

	for _, rawRound := range strings.Split(parts[1], "; ") {
		for _, record := range strings.Split(rawRound, ", ") {
			recordParts := strings.Split(record, " ")
			if len(recordParts) != 2 {
				return nil, fmt.Errorf("cannot split record '%s' on cound and color", record)
			}

			count, err := strconv.ParseUint(recordParts[0], 10, 8)
			if err != nil {
				return nil, fmt.Errorf("cannot parse '%s' as cubes cound: %w", recordParts[0], err)
			}

			switch recordParts[1] {
			default:
				continue
			case "blue":
				result.Blue = max(result.Blue, uint8(count))
			case "green":
				result.Green = max(result.Green, uint8(count))
			case "red":
				result.Red = max(result.Red, uint8(count))
			}
		}
	}

	return result, nil
}

func (m Minimum) Power() uint {
	return uint(m.Blue) * uint(m.Green) * uint(m.Red)
}

func GetPowerSum(r io.Reader) (uint, error) {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanLines)

	var result uint
	for scanner.Scan() {
		line := scanner.Text()
		game, err := ParseMinimum(line)
		if err != nil {
			return 0, fmt.Errorf("cannot parse game from line '%s': %w", line, err)
		}

		result += game.Power()
	}

	return result, nil
}
