package part2_test

import (
	"fmt"
	"log"
	"strings"
	"testing"

	"github.com/google/go-cmp/cmp"

	"source.toby3d.me/toby3d/adventofcode/internal/day2/part2"
)

func TestParseMinimum(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		input  string
		expect part2.Minimum
	}{
		"1": {
			input:  "Game 1: 3 blue, 4 red; 1 red, 2 green, 6 blue; 2 green",
			expect: part2.Minimum{Red: 4, Green: 2, Blue: 6},
		},
		"2": {
			input:  "Game 2: 1 blue, 2 green; 3 green, 4 blue, 1 red; 1 green, 1 blue",
			expect: part2.Minimum{Red: 1, Green: 3, Blue: 4},
		},
		"3": {
			input:  "Game 3: 8 green, 6 blue, 20 red; 5 blue, 4 red, 13 green; 5 green, 1 red",
			expect: part2.Minimum{Red: 20, Green: 13, Blue: 6},
		},
		"4": {
			input:  "Game 4: 1 green, 3 red, 6 blue; 3 green, 6 red; 3 green, 15 blue, 14 red",
			expect: part2.Minimum{Red: 14, Green: 3, Blue: 15},
		},
		"5": {
			input:  "Game 5: 6 red, 1 blue, 3 green; 2 blue, 1 red, 2 green",
			expect: part2.Minimum{Red: 6, Green: 3, Blue: 2},
		},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			actual, err := part2.ParseMinimum(tc.input)
			if err != nil {
				t.Fatal(err)
			}

			if diff := cmp.Diff(*actual, tc.expect); diff != "" {
				t.Error(diff)
			}
		})
	}
}

func TestMinimum_Power(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		input  part2.Minimum
		expect uint
	}{
		"1": {input: part2.Minimum{Red: 4, Green: 2, Blue: 6}, expect: 48},
		"2": {input: part2.Minimum{Red: 1, Green: 3, Blue: 4}, expect: 12},
		"3": {input: part2.Minimum{Red: 20, Green: 13, Blue: 6}, expect: 1560},
		"4": {input: part2.Minimum{Red: 14, Green: 3, Blue: 15}, expect: 630},
		"5": {input: part2.Minimum{Red: 6, Green: 3, Blue: 2}, expect: 36},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			if actual := tc.input.Power(); actual != tc.expect {
				t.Errorf("want %d, got %d", tc.expect, actual)
			}
		})
	}
}

func Example() {
	input := "Game 1: 3 blue, 4 red; 1 red, 2 green, 6 blue; 2 green\n" +
		"Game 2: 1 blue, 2 green; 3 green, 4 blue, 1 red; 1 green, 1 blue\n" +
		"Game 3: 8 green, 6 blue, 20 red; 5 blue, 4 red, 13 green; 5 green, 1 red\n" +
		"Game 4: 1 green, 3 red, 6 blue; 3 green, 6 red; 3 green, 15 blue, 14 red\n" +
		"Game 5: 6 red, 1 blue, 3 green; 2 blue, 1 red, 2 green"

	result, err := part2.GetPowerSum(strings.NewReader(input))
	if err != nil {
		log.Fatalln(err)
	}

	// The power of a set of cubes is equal to the numbers of red, green,
	// and blue cubes multiplied together. The power of the minimum set of
	// cubes in game 1 is 48. In games 2-5 it was 12, 1560, 630, and 36,
	// respectively. Adding up these five powers produces the sum 2286.
	fmt.Println(result)
	// Output: 2286
}
