package part1_test

import (
	"fmt"
	"strings"
	"testing"

	"source.toby3d.me/toby3d/adventofcode/internal/day3/part1"
)

func TestGetPartNumbersSum(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		input  string
		expect uint
	}{
		// NOTE(toby3d): thanks, Calcifer777!
		// https://old.reddit.com/r/adventofcode/comments/189m7uu/2023_day_3_part_1_okay_then/kbt4tae/
		"center": {
			input: "...12\n" +
				"12*..",
			expect: 24,
		},
		// NOTE(toby3d): thanks, i_have_no_biscuits!
		// https://old.reddit.com/r/adventofcode/comments/189q9wv/2023_day_3_another_sample_grid_to_use/
		"ultimate1": {
			input: "12.......*..\n" +
				"+.........34\n" +
				".......-12..\n" +
				"..78........\n" +
				"..*....60...\n" +
				"78..........\n" +
				".......23...\n" +
				"....90*12...\n" +
				"............\n" +
				"2.2......12.\n" +
				".*.........*\n" +
				"1.1.......56",
			expect: 413,
		},
		"ultimate2": {
			input: "12.......*..\n" +
				"+.........34\n" +
				".......-12..\n" +
				"..78........\n" +
				"..*....60...\n" +
				"78.........9\n" +
				".5.....23..$\n" +
				"8...90*12...\n" +
				"............\n" +
				"2.2......12.\n" +
				".*.........*\n" +
				"1.1..503+.56",
			expect: 925,
		},
		// NOTE(toby3d): thanks, IsatisCrucifer!
		// https://old.reddit.com/r/adventofcode/comments/189q9wv/2023_day_3_another_sample_grid_to_use/kbt0vh8/
		"search": {
			input: "........\n" +
				".24..4..\n" +
				"......*.",
			expect: 4,
		},
		// NOTE(toby3d): thanks, i_have_no_biscuits!
		// https://old.reddit.com/r/adventofcode/comments/189q9wv/2023_day_3_another_sample_grid_to_use/kbt2yp5/
		"same": {
			input: "....................\n" +
				"..-52..52-..52..52..\n" +
				"..................-.",
			expect: 156,
		},
		// NOTE(toby3d): thanks, musifter!
		// https://old.reddit.com/r/adventofcode/comments/189q9wv/2023_day_3_another_sample_grid_to_use/kbsrno0/
		"gears": {
			input: ".......5......\n" +
				"..7*..*.......\n" +
				"...*13*.......\n" +
				".......15.....",
			expect: 40,
		},
		// NOTE(toby3d): thanks, Martins31!
		// https://old.reddit.com/r/adventofcode/comments/189q9wv/2023_day_3_another_sample_grid_to_use/kbtiucy/
		"space": {
			input: ".......5......\n" +
				"..7*..*.....4*\n" +
				"...*13*......9\n" +
				".......15.....\n" +
				"..............\n" +
				"..............\n" +
				"..............\n" +
				"..............\n" +
				"..............\n" +
				"..............\n" +
				"21............\n" +
				"...*9.........",
			expect: 62,
		},
		// NOTE(toby3d): thanks, 1234abcdcba4321!
		// https://old.reddit.com/r/adventofcode/comments/18a9pnl/2023_day_3_part_1_go_need_a_fresh_pair_of_eyes/kbwicj0/
		"zero": {
			input: "...\n" +
				"1.*\n" +
				"...",
			expect: 0,
		},
		// NOTE(toby3d): thanks, 1234abcdcba4321!
		// https://old.reddit.com/r/adventofcode/comments/18a2yir/year_2023_day_3_part_1_are_part_numbers_mutually/kbv5q64/
		"ones": {
			input: "1..\n" +
				"*.1\n" +
				"1..",
			expect: 2,
		},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			if actual := part1.GetPartNumbersSum(strings.NewReader(tc.input)); actual != tc.expect {
				t.Errorf("want %d, got %d", tc.expect, actual)
			}
		})
	}
}

func Example() {
	// The engine schematic (your puzzle input) consists of a visual
	// representation of the engine. There are lots of numbers and symbols
	// you don't really understand, but apparently any number adjacent to a
	// symbol, even diagonally, is a "part number" and should be included in
	// your sum. (Periods (.) do not count as a symbol.)
	input := "467..114..\n" +
		"...*......\n" +
		"..35..633.\n" +
		"......#...\n" +
		"617*......\n" +
		".....+.58.\n" +
		"..592.....\n" +
		"......755.\n" +
		"...$.*....\n" +
		".664.598.."

	fmt.Println(part1.GetPartNumbersSum(strings.NewReader(input)))
	// Output: 4361
}
