package part2

import (
	"bufio"
	"io"
	"strconv"
)

type (
	EngineMap [][]Point
	Point     byte

	Number struct {
		value []byte
		minX  int
		maxX  int
		y     int
	}

	Gear struct {
		x, y int
	}
)

func NewNumber(y int) *Number {
	return &Number{
		minX:  0,
		maxX:  0,
		y:     y,
		value: make([]byte, 0),
	}
}

func (n *Number) Write(x int, p Point) {
	n.value = append(n.value, byte(p))
	n.maxX = x
}

func (n Number) IsZero() bool {
	return len(n.value) == 0
}

func (n *Number) Move(x int) {
	n.minX = x
	n.maxX = x
}

func (n Number) Uint() uint {
	result, _ := strconv.ParseUint(string(n.value), 10, 64)

	return uint(result)
}

func (p Point) IsGear() bool {
	return p == '*'
}

func (p Point) IsDigit() bool {
	switch p {
	default:
		return false
	case '0', '1', '2', '3', '4', '5', '6', '7', '8', '9':
		return true
	}
}

func (p Point) IsSymbol() bool {
	switch p {
	default:
		return true
	case '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.':
		return false
	}
}

func (p Point) IsEmpty() bool {
	return p == '.'
}

func (em EngineMap) HasGearAround(n Number) *Gear {
	minx := n.minX - 1
	if minx < 0 {
		minx = 0
	}

	maxx := n.maxX + 1
	if maxx >= len(em[0]) {
		maxx = len(em[0]) - 1
	}

	miny := n.y - 1
	if miny < 0 {
		miny = 0
	}

	maxy := n.y + 1
	if maxy >= len(em) {
		maxy = len(em) - 1
	}

	for y := miny; y <= maxy; y++ {
		for x := minx; x <= maxx; x++ {
			if !em[y][x].IsGear() {
				continue
			}

			return &Gear{
				x: x,
				y: y,
			}
		}
	}

	return nil
}

func ParseEngineMap(r io.Reader) EngineMap {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanLines)

	engineMap := make(EngineMap, 0)
	y := 0

	for scanner.Scan() {
		line := scanner.Bytes()
		engineMap = append(engineMap, make([]Point, len(line)))

		for x := range line {
			engineMap[y][x] = Point(line[x])
		}

		y++
	}

	return engineMap
}

func (em EngineMap) GetPartNumbers() []Number {
	numbers := make([]Number, 0)

	for y := range em {
		n := NewNumber(y)

		for x := range em[y] {
			p := em[y][x]

			if p.IsDigit() {
				if n.IsZero() {
					n.Move(x)
				}

				n.Write(x, p)

				continue
			}

			if n.IsZero() {
				continue
			}

			numbers = append(numbers, *n)
			n = NewNumber(y)
		}

		if !n.IsZero() {
			numbers = append(numbers, *n)
		}
	}

	return numbers
}

func (em EngineMap) GetGearRatiosSum() uint64 {
	// NOTE(toby3d): Numbers are harder to find and take up more space than
	// symbols. So, instead of looking for gears all over the map, I will
	// limit myself to looking for gears around numbers and assign them to
	// the same gear point.
	gears := make(map[Gear][]Number)

	for _, number := range em.GetPartNumbers() {
		gear := em.HasGearAround(number)
		if gear == nil {
			continue
		}

		if _, ok := gears[*gear]; !ok {
			gears[*gear] = make([]Number, 0)
		}

		gears[*gear] = append(gears[*gear], number)
	}

	var result uint64
	for _, numbers := range gears {
		if len(numbers) != 2 {
			continue
		}

		result += uint64(numbers[0].Uint()) * uint64(numbers[1].Uint())
	}

	return result
}
