package part1_test

import (
	"fmt"
	"log"
	"strings"
	"testing"

	"github.com/google/go-cmp/cmp"

	"source.toby3d.me/toby3d/adventofcode/internal/day4/part1"
)

func TestFindWinningNumbers(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		numbers, winning []uint8
		expect           []uint8
	}{
		"1": {
			numbers: []uint8{41, 48, 83, 86, 17},
			winning: []uint8{83, 86, 6, 31, 17, 9, 48, 53},
			expect:  []uint8{48, 83, 86, 17},
		},
		"2": {
			numbers: []uint8{13, 32, 20, 16, 61},
			winning: []uint8{61, 30, 68, 82, 17, 32, 24, 19},
			expect:  []uint8{32, 61},
		},
		"3": {
			numbers: []uint8{1, 21, 53, 59, 44},
			winning: []uint8{69, 82, 63, 72, 16, 21, 14, 1},
			expect:  []uint8{1, 21},
		},
		"4": {
			numbers: []uint8{41, 92, 73, 84, 69},
			winning: []uint8{59, 84, 76, 51, 58, 5, 54, 83},
			expect:  []uint8{84},
		},
		"5": {
			numbers: []uint8{87, 83, 26, 28, 32},
			winning: []uint8{88, 30, 70, 12, 93, 22, 82, 36},
			expect:  make([]uint8, 0),
		},
		"6": {
			numbers: []uint8{31, 18, 13, 56, 72},
			winning: []uint8{74, 77, 10, 23, 35, 67, 36, 11},
			expect:  make([]uint8, 0),
		},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			actual := part1.FindWinningNumbers(tc.numbers, tc.winning)
			if diff := cmp.Diff(actual, tc.expect); diff != "" {
				t.Error(diff)
			}
		})
	}
}

func Example() {
	// The Elf leads you over to the pile of colorful cards. There, you
	// discover dozens of scratchcards, all with their opaque covering
	// already scratched off. Picking one up, it looks like each card has
	// two lists of numbers separated by a vertical bar (|): a list of
	// winning numbers and then a list of numbers you have. You organize the
	// information into a table (your puzzle input).
	//
	// As far as the Elf has been able to figure out, you have to figure out
	// which of the numbers you have appear in the list of winning numbers.
	// The first match makes the card worth one point and each match after
	// the first doubles the point value of that card.
	input := "Card 1: 41 48 83 86 17 | 83 86  6 31 17  9 48 53\n" +
		"Card 2: 13 32 20 16 61 | 61 30 68 82 17 32 24 19\n" +
		"Card 3:  1 21 53 59 44 | 69 82 63 72 16 21 14  1\n" +
		"Card 4: 41 92 73 84 69 | 59 84 76 51 58  5 54 83\n" +
		"Card 5: 87 83 26 28 32 | 88 30 70 12 93 22 82 36\n" +
		"Card 6: 31 18 13 56 72 | 74 77 10 23 35 67 36 11"

	total, err := part1.GetTotalWorthPoints(strings.NewReader(input))
	if err != nil {
		log.Fatalln(err)
	}

	fmt.Println(total)
	// Output: 13
}
