package part2

import (
	"bufio"
	"fmt"
	"io"
	"slices"
	"strconv"
	"strings"
)

type Card struct {
	Instances int
	Wins      int
}

func FindWinningNumbers(numbers, winning []uint8) []uint8 {
	result := make([]uint8, 0, len(numbers))

	for i := range numbers {
		if !slices.Contains(winning, numbers[i]) {
			continue
		}

		result = append(result, numbers[i])
	}

	return result
}

func GetTotalScratchcards(r io.Reader) (uint, error) {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanLines)

	cards := make([]Card, 0) // [cardNumber]matches

	for scanner.Scan() {
		line := scanner.Text()

		parts := strings.Split(line, ":")
		if len(parts) != 2 {
			return 0, fmt.Errorf("expect string in format 'Game 99: 1 2 3... | 1 2 3...', got '%s'", line)
		}

		parts = strings.Split(parts[1], " | ")
		if len(parts) != 2 {
			return 0, fmt.Errorf("expect numbers in format '1 2 3... | 1 2 3...', got '%s'", line)
		}

		var numbers [2][]uint8

		for i := range parts {
			numbers[i] = make([]uint8, 0)

			for _, rawNumber := range strings.Fields(parts[i]) {
				number, err := strconv.ParseUint(rawNumber, 10, 8)
				if err != nil {
					return 0, fmt.Errorf("expect number, got '%s'", rawNumber)
				}

				numbers[i] = append(numbers[i], uint8(number))
			}
		}

		cards = append(cards, Card{
			Instances: 1,
			Wins:      len(FindWinningNumbers(numbers[0], numbers[1])),
		})
	}

	return uint(WinScratchcards(cards)), nil
}

func WinScratchcards(cards []Card) int {
	var count int

	for i := range cards {
		count += cards[i].Instances

		if cards[i].Wins == 0 {
			continue
		}

		for j := i + cards[i].Wins; i < j; j-- {
			cards[j].Instances += cards[i].Instances
		}
	}

	return count
}
