package part2_test

import (
	"fmt"
	"log"
	"strings"
	"testing"

	"github.com/google/go-cmp/cmp"

	"source.toby3d.me/toby3d/adventofcode/internal/day4/part2"
)

func TestFindWinningNumbers(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		numbers, winning []uint8
		expect           []uint8
	}{
		"1": {
			numbers: []uint8{41, 48, 83, 86, 17},
			winning: []uint8{83, 86, 6, 31, 17, 9, 48, 53},
			expect:  []uint8{48, 83, 86, 17},
		},
		"2": {
			numbers: []uint8{13, 32, 20, 16, 61},
			winning: []uint8{61, 30, 68, 82, 17, 32, 24, 19},
			expect:  []uint8{32, 61},
		},
		"3": {
			numbers: []uint8{1, 21, 53, 59, 44},
			winning: []uint8{69, 82, 63, 72, 16, 21, 14, 1},
			expect:  []uint8{1, 21},
		},
		"4": {
			numbers: []uint8{41, 92, 73, 84, 69},
			winning: []uint8{59, 84, 76, 51, 58, 5, 54, 83},
			expect:  []uint8{84},
		},
		"5": {
			numbers: []uint8{87, 83, 26, 28, 32},
			winning: []uint8{88, 30, 70, 12, 93, 22, 82, 36},
			expect:  make([]uint8, 0),
		},
		"6": {
			numbers: []uint8{31, 18, 13, 56, 72},
			winning: []uint8{74, 77, 10, 23, 35, 67, 36, 11},
			expect:  make([]uint8, 0),
		},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			actual := part2.FindWinningNumbers(tc.numbers, tc.winning)
			if diff := cmp.Diff(actual, tc.expect); diff != "" {
				t.Error(diff)
			}
		})
	}
}

func Example() {
	// Specifically, you win copies of the scratchcards below the winning
	// card equal to the number of matches. So, if card 10 were to have 5
	// matching numbers, you would win one copy each of cards 11, 12, 13,
	// 14, and 15.
	//
	// Copies of scratchcards are scored like normal scratchcards and have
	// the same card number as the card they copied. So, if you win a copy
	// of card 10 and it has 5 matching numbers, it would then win a copy of
	// the same cards that the original card 10 won: cards 11, 12, 13, 14,
	// and 15. This process repeats until none of the copies cause you to
	// win any more cards. (Cards will never make you copy a card past the
	// end of the table.)
	input := "Card 1: 41 48 83 86 17 | 83 86  6 31 17  9 48 53\n" +
		"Card 2: 13 32 20 16 61 | 61 30 68 82 17 32 24 19\n" +
		"Card 3:  1 21 53 59 44 | 69 82 63 72 16 21 14  1\n" +
		"Card 4: 41 92 73 84 69 | 59 84 76 51 58  5 54 83\n" +
		"Card 5: 87 83 26 28 32 | 88 30 70 12 93 22 82 36\n" +
		"Card 6: 31 18 13 56 72 | 74 77 10 23 35 67 36 11"

	total, err := part2.GetTotalScratchcards(strings.NewReader(input))
	if err != nil {
		log.Fatalln(err)
	}

	fmt.Println(total)
	// Output: 30
}
