package part2_test

import (
	"fmt"
	"log"
	"strings"
	"testing"

	"github.com/google/go-cmp/cmp"
	"github.com/google/go-cmp/cmp/cmpopts"

	"source.toby3d.me/toby3d/adventofcode/internal/day5/part2"
)

var filter = cmpopts.IgnoreUnexported(part2.Map{}, part2.Range{})

func TestParseSeeds(t *testing.T) {
	t.Parallel()

	actual, err := part2.ParseSeeds("seeds: 79 14 55 13")
	if err != nil {
		t.Fatal(err)
	}

	expect := []part2.Seed{
		{Start: 79, Length: 14},
		{Start: 55, Length: 13},
	}
	if diff := cmp.Diff(actual, expect); diff != "" {
		t.Error(diff)
	}
}

// The values on the initial seeds: line come in pairs. Within each pair, the
// first value is the start of the range and the second value is the length of
// the range.
func Example() {
	input := "seeds: 79 14 55 13\n" +
		"\n" +
		"seed-to-soil map:\n" +
		"50 98 2\n" +
		"52 50 48\n" +
		"\n" +
		"soil-to-fertilizer map:\n" +
		"0 15 37\n" +
		"37 52 2\n" +
		"39 0 15\n" +
		"\n" +
		"fertilizer-to-water map:\n" +
		"49 53 8\n" +
		"0 11 42\n" +
		"42 0 7\n" +
		"57 7 4\n" +
		"\n" +
		"water-to-light map:\n" +
		"88 18 7\n" +
		"18 25 70\n" +
		"\n" +
		"light-to-temperature map:\n" +
		"45 77 23\n" +
		"81 45 19\n" +
		"68 64 13\n" +
		"\n" +
		"temperature-to-humidity map:\n" +
		"0 69 1\n" +
		"1 0 69\n" +
		"\n" +
		"humidity-to-location map:\n" +
		"60 56 37\n" +
		"56 93 4"

	almanac, err := part2.ParseAlmanac(strings.NewReader(input))
	if err != nil {
		log.Fatalln(err)
	}

	fmt.Println(almanac.FindLowestLocation())
	// Output: 46
}
