package part1

import (
	"bufio"
	"fmt"
	"io"
	"strconv"
	"strings"
)

type (
	Race struct {
		Time     uint // milliseconds
		Distance uint // millimeters
	}

	Races []Race
)

func RunBoat(limit, hold uint) uint {
	if hold == 0 || hold == limit {
		return 0
	}

	var distance uint
	var speed uint = hold

	// TODO(toby3d): something useful MUST be in the math package, but I'm
	// stupid.
	for i := limit - hold; 0 < i; i-- {
		distance += speed
	}

	return distance
}

func (r Race) CountWinningStrategies() uint {
	var count uint
	for i := uint(1); i < r.Time-1; i++ {
		if distance := RunBoat(r.Time, i); r.Distance < distance {
			count++
		}
	}

	return count
}

func (r Races) Power() uint {
	power := r[0].CountWinningStrategies()

	for i := 1; i < len(r); i++ {
		power *= r[i].CountWinningStrategies()
	}

	return power
}

func ParseDocument(r io.Reader) (Races, error) {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanLines)

	result := make(Races, 0)

	for scanner.Scan() {
		line := scanner.Text()
		isTime := strings.HasPrefix(line, "Time")
		isDistance := strings.HasPrefix(line, "Distance")

		if !isTime && !isDistance {
			continue
		}

		data := strings.Fields(line[strings.IndexRune(line, ':')+1:])
		if len(result) == 0 {
			result = make(Races, len(data))
		}

		for i := range data {
			parsed, err := strconv.ParseUint(data[i], 10, 64)
			if err != nil {
				return nil, fmt.Errorf("cannot parse '%s' as number: %w", data[i], err)
			}

			switch {
			case isTime:
				result[i].Time = uint(parsed)
			case isDistance:
				result[i].Distance = uint(parsed)
			}
		}
	}

	return result, nil
}
