package part1_test

import (
	"fmt"
	"log"
	"strings"
	"testing"

	"source.toby3d.me/toby3d/adventofcode/internal/day6/part1"
)

func TestRunBoat(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		input  uint
		expect uint
	}{
		"nothing": {0, 0},
		"1":       {1, 6},
		"2":       {2, 10},
		"3":       {3, 12},
		"4":       {4, 12},
		"5":       {5, 10},
		"6":       {6, 6},
		"wait":    {7, 0},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			if actual := part1.RunBoat(7, tc.input); actual != tc.expect {
				t.Errorf("want %d, got %d", tc.expect, actual)
			}
		})
	}
}

func TestRace_CountWinStrategies(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		input  part1.Race
		expect uint
	}{
		"1": {input: part1.Race{Time: 7, Distance: 9}, expect: 4},
		"2": {input: part1.Race{Time: 15, Distance: 40}, expect: 8},
		"3": {input: part1.Race{Time: 30, Distance: 200}, expect: 9},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			if actual := tc.input.CountWinningStrategies(); actual != tc.expect {
				t.Errorf("want %d, got %d", tc.expect, actual)
			}
		})
	}
}

// As part of signing up, you get a sheet of paper (your puzzle input) that
// lists the time allowed for each race and also the best distance ever recorded
// in that race. To guarantee you win the grand prize, you need to make sure you
// go farther in each race than the current record holder.
//
// The organizer brings you over to the area where the boat races are held. The
// boats are much smaller than you expected - they're actually toy boats, each
// with a big button on top. Holding down the button charges the boat, and
// releasing the button allows the boat to move. Boats move faster if their
// button was held longer, but time spent holding the button counts against the
// total race time. You can only hold the button at the start of the race, and
// boats don't move until the button is released.
func Example() {
	input := "Time:      7  15   30\n" +
		"Distance:  9  40  200"

	races, err := part1.ParseDocument(strings.NewReader(input))
	if err != nil {
		log.Fatalln(err)
	}

	fmt.Println(races.Power())
	// Output: 288
}
