package part2

import (
	"bufio"
	"fmt"
	"io"
	"strconv"
	"strings"
)

type (
	Race struct {
		Time     uint // milliseconds
		Distance uint // millimeters
	}
)

func RunBoat(limit, hold uint) uint {
	if hold == 0 || hold == limit {
		return 0
	}

	var distance uint = hold * (limit - hold)

	return distance
}

func (r Race) CountWinningStrategies() uint {
	var count uint
	for i := uint(1); i < r.Time-1; i++ {
		if distance := RunBoat(r.Time, i); r.Distance < distance {
			count++
		}
	}

	return count
}

func ParseDocument(r io.Reader) (*Race, error) {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanLines)

	result := new(Race)

	for scanner.Scan() {
		line := scanner.Text()
		isTime := strings.HasPrefix(line, "Time")
		isDistance := strings.HasPrefix(line, "Distance")

		if !isTime && !isDistance {
			continue
		}

		value := strings.Join(strings.Fields(line[strings.IndexRune(line, ':')+1:]), "")

		parsed, err := strconv.ParseUint(value, 10, 64)
		if err != nil {
			return nil, fmt.Errorf("cannot parse '%s' as number: %w", value, err)
		}

		switch {
		case isTime:
			result.Time = uint(parsed)
		case isDistance:
			result.Distance = uint(parsed)
		}
	}

	return result, nil
}
