package part1_test

import (
	"fmt"
	"log"
	"strings"
	"testing"

	"source.toby3d.me/toby3d/adventofcode/internal/day7/part1"
)

func TestNewHand(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		input  [5]part1.Card
		expect part1.HandType
	}{
		"FiveOfTheKind": {
			input:  [5]part1.Card{part1.CardA, part1.CardA, part1.CardA, part1.CardA, part1.CardA},
			expect: part1.HandTypeFiveOfTheKind,
		},
		"FourOfAKind": {
			input:  [5]part1.Card{part1.CardA, part1.CardA, part1.Card8, part1.CardA, part1.CardA},
			expect: part1.HandTypeFourOfAKind,
		},
		"FullHouse": {
			input:  [5]part1.Card{part1.Card2, part1.Card3, part1.Card3, part1.Card3, part1.Card2},
			expect: part1.HandTypeFullHouse,
		},
		"ThreeOfAKind": {
			input:  [5]part1.Card{part1.CardT, part1.CardT, part1.CardT, part1.Card9, part1.Card8},
			expect: part1.HandTypeThreeOfAKind,
		},
		"TwoPair": {
			input:  [5]part1.Card{part1.Card2, part1.Card3, part1.Card4, part1.Card3, part1.Card2},
			expect: part1.HandTypeTwoPair,
		},
		"OnePair": {
			input:  [5]part1.Card{part1.CardA, part1.Card2, part1.Card3, part1.CardA, part1.Card4},
			expect: part1.HandTypeOnePair,
		},
		"HighCard": {
			input:  [5]part1.Card{part1.Card2, part1.Card3, part1.Card4, part1.Card5, part1.Card6},
			expect: part1.HandTypeHighCard,
		},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			if actual := part1.NewHand(tc.input).Type(); actual != tc.expect {
				t.Errorf("want %v, got %v", tc.expect, actual)
			}
		})
	}
}

func TestHand_WinAgainst(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		first, second part1.Hand
	}{
		"1": {
			part1.NewHand([5]part1.Card{part1.Card3, part1.Card3, part1.Card3, part1.Card3, part1.Card2}),
			part1.NewHand([5]part1.Card{part1.Card2, part1.CardA, part1.CardA, part1.CardA, part1.CardA}),
		},
		"2": {
			part1.NewHand([5]part1.Card{part1.Card7, part1.Card7, part1.Card8, part1.Card8, part1.Card8}),
			part1.NewHand([5]part1.Card{part1.Card7, part1.Card7, part1.Card7, part1.Card8, part1.Card8}),
		},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			if actual := tc.first.IsWinAgainst(tc.second); !actual {
				t.Error("expect true, got false")
			}
		})
	}
}

func Example() {
	// To play Camel Cards, you are given a list of hands and their
	// corresponding bid (your puzzle input).
	input := "32T3K 765\n" +
		"T55J5 684\n" +
		"KK677 28\n" +
		"KTJJT 220\n" +
		"QQQJA 483"

	camelCards, err := part1.ParseCamelCards(strings.NewReader(input))
	if err != nil {
		log.Fatalln(err)
	}

	fmt.Println(camelCards.CountTotalWinnings())
	// Output: 6440
}
