package part2_test

import (
	"fmt"
	"log"
	"strings"
	"testing"

	"source.toby3d.me/toby3d/adventofcode/internal/day7/part2"
)

func TestNewHand(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		input  [5]part2.Card
		expect part2.HandType
	}{
		"HighCard2OnePair": {
			input:  [5]part2.Card{part2.CardJ, part2.Card2, part2.Card3, part2.Card4, part2.Card5},
			expect: part2.HandTypeOnePair,
		},
		// NOTE(toby3d): TwoPair is weaken then ThreeOfKind, Joker must
		// mimic the best possible combination.
		"OnePair2ThreeOfKind": {
			input:  [5]part2.Card{part2.CardJ, part2.Card2, part2.Card2, part2.Card3, part2.Card4},
			expect: part2.HandTypeThreeOfAKind,
		},
		// NOTE(toby3d): ThreeOfKind is weaken then FullHouse, Joker
		// must mimic the best possible combination.
		"TwoPair2FullHouse": {
			input:  [5]part2.Card{part2.CardJ, part2.Card2, part2.Card2, part2.Card3, part2.Card3},
			expect: part2.HandTypeFullHouse,
		},
		// NOTE(toby3d): FullHouse is weaken then FourOfKind, Joker must
		// mimic the best possible combination.
		"ThreeOfAKind2FourOfAKind": {
			input:  [5]part2.Card{part2.CardJ, part2.Card2, part2.Card2, part2.Card2, part2.Card3},
			expect: part2.HandTypeFourOfAKind,
		},
		"ThreeOfAKind2FourOfAKindExample": {
			input:  [5]part2.Card{part2.CardQ, part2.CardJ, part2.CardJ, part2.CardQ, part2.Card2},
			expect: part2.HandTypeFourOfAKind,
		},
		"FullHouse2FiveOfTheKind": {
			input:  [5]part2.Card{part2.CardJ, part2.CardJ, part2.Card2, part2.Card2, part2.Card2},
			expect: part2.HandTypeFiveOfTheKind,
		},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			if actual := part2.NewHand(tc.input).Type(); actual != tc.expect {
				t.Errorf("want %v, got %v", tc.expect, actual)
			}
		})
	}
}

func Example() {
	input := "32T3K 765\n" +
		"T55J5 684\n" +
		"KK677 28\n" +
		"KTJJT 220\n" +
		"QQQJA 483"

	camelCards, err := part2.ParseCamelCards(strings.NewReader(input))
	if err != nil {
		log.Fatalln(err)
	}

	fmt.Println(camelCards.CountTotalWinnings())
	// Output: 5905
}
