package part1

import (
	"bufio"
	"io"
)

type (
	NetworkMap struct {
		Forks      map[Node]Fork
		Directions []Direction
	}

	Fork struct {
		Left  Node
		Right Node
	}

	Node [3]byte

	Direction byte
)

var (
	NodeUnd   Node = [3]byte{0, 0, 0}
	NodeStart Node = [3]byte{'A', 'A', 'A'}
	NodeEnd   Node = [3]byte{'Z', 'Z', 'Z'}
)

var (
	DirectionLeft  Direction = 'L'
	DirectionRight Direction = 'R'
)

func (f Fork) Peek(d Direction) Node {
	switch d {
	default:
		return NodeUnd
	case DirectionLeft:
		return f.Left
	case DirectionRight:
		return f.Right
	}
}

func ParseNetworkMap(r io.Reader) *NetworkMap {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanLines)

	networkMap := &NetworkMap{
		Directions: make([]Direction, 0),
		Forks:      make(map[Node]Fork),
	}

	scanner.Scan()
	for _, b := range scanner.Text() {
		networkMap.Directions = append(networkMap.Directions, Direction(b))
	}

	for scanner.Scan() {
		line := scanner.Text()

		if len(line) == 0 {
			continue
		}

		node := Node([3]byte{line[0], line[1], line[2]})
		leftNode := Node([3]byte{line[7], line[8], line[9]})
		rightNode := Node([3]byte{line[12], line[13], line[14]})
		networkMap.Forks[node] = Fork{
			Left:  leftNode,
			Right: rightNode,
		}
	}

	return networkMap
}

func (nm NetworkMap) CountSteps() uint64 {
	var steps uint64
	node := NodeStart

	for i := 0; node != NodeEnd; i++ {
		i = i % len(nm.Directions)
		node = nm.Forks[node].Peek(nm.Directions[i])
		steps++
	}

	return steps
}
