package part2

import (
	"bufio"
	"io"
	"sync"
)

type (
	NetworkMap struct {
		Starts     []Node
		Forks      map[Node]Fork
		Directions []Direction
	}

	Fork struct {
		Left  Node
		Right Node
	}

	Ghost struct {
		Steps    uint64
		Position Node
	}

	Node [3]byte

	Direction byte
)

var NodeUnd Node = [3]byte{0, 0, 0}

var (
	DirectionLeft  Direction = 'L'
	DirectionRight Direction = 'R'
)

func (n Node) IsStart() bool {
	return n[2] == 'A'
}

func (n Node) IsEnd() bool {
	return n[2] == 'Z'
}

func (f Fork) Peek(d Direction) Node {
	switch d {
	default:
		return NodeUnd
	case DirectionLeft:
		return f.Left
	case DirectionRight:
		return f.Right
	}
}

func ParseNetworkMap(r io.Reader) *NetworkMap {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanLines)

	networkMap := &NetworkMap{
		Directions: make([]Direction, 0),
		Forks:      make(map[Node]Fork),
	}

	scanner.Scan()
	for _, b := range scanner.Text() {
		networkMap.Directions = append(networkMap.Directions, Direction(b))
	}

	for scanner.Scan() {
		line := scanner.Text()

		if len(line) == 0 {
			continue
		}

		node := Node([3]byte{line[0], line[1], line[2]})

		if node.IsStart() {
			networkMap.Starts = append(networkMap.Starts, node)
		}

		leftNode := Node([3]byte{line[7], line[8], line[9]})
		rightNode := Node([3]byte{line[12], line[13], line[14]})
		networkMap.Forks[node] = Fork{
			Left:  leftNode,
			Right: rightNode,
		}

	}

	return networkMap
}

func (nm NetworkMap) moveGhost(g *Ghost) {
	for {
		g.Position = nm.Forks[g.Position].Peek(nm.Directions[int(g.Steps)%len(nm.Directions)])
		g.Steps++

		if !g.Position.IsEnd() {
			continue
		}

		break
	}
}

func (nm NetworkMap) CountSteps() uint64 {
	ghosts := make([]*Ghost, len(nm.Starts))
	wg := new(sync.WaitGroup)

	for i := range ghosts {
		ghosts[i] = &Ghost{
			Position: nm.Starts[i],
			Steps:    0,
		}

		wg.Add(1)
		go func(ghost *Ghost) {
			nm.moveGhost(ghost)
			wg.Done()
		}(ghosts[i])
	}

	wg.Wait()

	steps := make([]uint64, len(ghosts))
	for i := range ghosts {
		steps[i] = ghosts[i].Steps
	}

	return lcm(steps...)
}

// NOTE(toby3d): adapted version of the python code from here:
// https://www.geeksforgeeks.org/finding-lcm-two-array-numbers-without-using-gcd/
//
// Thanks /r/adventofcode for memes about this day puzzle. Today I know what the
// fuck is LCM mean.
func lcm(inputs ...uint64) uint64 {
	var maxNum uint64

	for i := range inputs {
		maxNum = max(maxNum, inputs[i])
	}

	var result, factor uint64 = 1, 2

	for factor <= maxNum {
		indexes := make([]int, 0, len(inputs))

		for i := range inputs {
			if inputs[i]%factor == 0 {
				indexes = append(indexes, i)
			}
		}

		if len(indexes) >= 2 {
			for i := range indexes {
				inputs[indexes[i]] = inputs[indexes[i]] / factor
			}

			result *= factor
		} else {
			factor++
		}
	}

	for i := range inputs {
		result *= inputs[i]
	}

	return result
}

func (d Direction) String() string {
	switch d {
	default:
		return ""
	case DirectionLeft:
		return "left"
	case DirectionRight:
		return "right"
	}
}

func (d Direction) GoString() string {
	return "part2.Direction(" + string(d) + ")"
}
