package part1

import (
	"bufio"
	"fmt"
	"io"
	"strconv"
	"strings"
)

type (
	Report  []History
	History [][]int64
)

func (h History) Predict() int64 {
	for i := len(h) - 1; 0 <= i; i-- {
		left := h[i][len(h[i])-1]

		var below int64 = 0
		if i < len(h)-1 {
			below = h[i+1][len(h[i+1])-1]
		}

		h[i] = append(h[i], left+below)
	}

	return h[0][len(h[0])-1]
}

func ParseReport(r io.Reader) (Report, error) {
	scanner := bufio.NewScanner(r)
	scanner.Split(bufio.ScanLines)

	report := make(Report, 0)

	for scanner.Scan() {
		line := scanner.Text()
		parts := strings.Fields(line)

		var prediction int
		history := make(History, 0)
		history = append(history, make([]int64, len(parts)))

		for i := range history[prediction] {
			var err error
			if history[prediction][i], err = strconv.ParseInt(parts[i], 10, 64); err != nil {
				return nil, fmt.Errorf("cannot parse history part '%s' as int64: %w", parts[i], err)
			}
		}

		var stop bool

		for !stop {
			prediction++
			history = append(history, make([]int64, len(history[prediction-1])-1))
			stop = true

			for i := 0; i < len(history[prediction]); i++ {
				history[prediction][i] = history[prediction-1][i+1] - history[prediction-1][i]
				if history[prediction][i] != 0 {
					stop = false
				}
			}
		}

		report = append(report, history)
	}

	return report, nil
}

func (r Report) Sum() int64 {
	var result int64
	for i := range r {
		result += r[i].Predict()
	}

	return result
}
