package part2_test

import (
	"fmt"
	"log"
	"strings"
	"testing"

	"source.toby3d.me/toby3d/adventofcode/internal/day9/part2"
)

func TestHistory_Predict(t *testing.T) {
	t.Parallel()

	for name, tc := range map[string]struct {
		input  part2.History
		expect int64
	}{
		"short": {input: part2.History{
			[]int64{0, 3, 6, 9, 12, 15},
			[]int64{3, 3, 3, 3, 3},
			[]int64{0, 0, 0, 0},
		}, expect: -3},
		"medium": {input: part2.History{
			[]int64{1, 3, 6, 10, 15, 21},
			[]int64{2, 3, 4, 5, 6},
			[]int64{1, 1, 1, 1},
			[]int64{0, 0, 0, 0},
		}, expect: 0},
		"long": {input: part2.History{
			[]int64{10, 13, 16, 21, 30, 45},
			[]int64{3, 3, 5, 9, 15},
			[]int64{0, 2, 4, 6},
			[]int64{2, 2, 2},
			[]int64{0, 0},
		}, expect: 5},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			if actual := tc.input.Predict(); actual != tc.expect {
				t.Errorf("want %d, got %d", tc.expect, actual)
			}
		})
	}
}

func Example() {
	input := "0 3 6 9 12 15\n" +
		"1 3 6 10 15 21\n" +
		"10 13 16 21 30 45"

	report, err := part2.ParseReport(strings.NewReader(input))
	if err != nil {
		log.Fatalln(err)
	}

	fmt.Println(report.Sum())
	// Output: 2
}
